@@ -1,5 +1,5 @@
 #!/bin/bash
-#  Copyright (C) 2000-2006 SWsoft. All rights reserved.
+#  Copyright (C) 2000-2007 SWsoft. All rights reserved.
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -30,8 +30,6 @@
 #
 VENET_DEV=venet0
 
-FAKEGATEWAY=191.255.255.1
-FAKEGATEWAYNET=191.255.255.0
 
 IFCFG_DIR=/etc/rc.d
 IFCFG=${IFCFG_DIR}/rc.inet1
@@ -42,7 +40,7 @@ function fix_rcinet1()
 
 	[ -f "${IFCFG}" ] || return 0
 	cp -fp ${IFCFG} ${IFCFG}.$$ || error "unable to create ${IFCFG}" ${VZ_FS_NO_DISK_SPACE}
-	sed -e "s/^GATEWAY=.*/GATEWAY=\"${FAKEGATEWAY}\"/" -e 's/^USE_DHCP=.*/\#USE_DHCP=\"\"/' -e 's/eth0/venet0/g' -e 's/^[\ \t]*\/sbin\/route add default gw .*/\t\/sbin\/route add -net 191.255.255.0\/24 dev venet0; \/sbin\/route add default gw \${GATEWAY} dev venet0/' < ${IFCFG} > ${IFCFG}.$$ && mv -f ${IFCFG}.$$ ${IFCFG}
+	sed -e "s/^GATEWAY=.*/GATEWAY=\"${FAKEGATEWAY}\"/" -e 's/^USE_DHCP=.*/\#USE_DHCP=\"\"/' -e 's/eth0/venet0/g' -e 's/^[\ \t]*\/sbin\/route add default gw .*/\t\/sbin\/route add -net '${FAKEGATEWAYNET}'\/24 dev venet0; \/sbin\/route add default gw \${GATEWAY} dev venet0/' < ${IFCFG} > ${IFCFG}.$$ && mv -f ${IFCFG}.$$ ${IFCFG}
 	rm -f ${IFCFG}.$$ >/dev/null 2>&1
 }
 
