@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000-2006 SWsoft. All rights reserved.
+ *  Copyright (C) 2000-2007 SWsoft. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -19,10 +19,18 @@
 #define	_VETH_H_
 
 #include "list.h"
+#include "dist.h"
 
 #define IFNAMSIZE	16
 #define ETH_ALEN	6
-#define VZNETCFG	"/usr/sbin/vznetcfg"
+#define MAC_SIZE	3*ETH_ALEN - 1
+#define VZNETCFG	SBINDIR "/vznetcfg"
+
+#define PROC_VETH	"/proc/vz/veth"
+
+#define SW_OUI		0x001851
+#define DEF_VETHNAME	"veth$VEID"
+
 
 /** Data structure for devices.
  */
@@ -35,12 +43,16 @@ typedef struct {
 	int addrlen_ve;			/**< VE device MAC address length. */
 	char dev_name_ve[IFNAMSIZE];	/**< device name in VE. */
 	int flags;
+	int active;
+	int configure;
 } veth_dev;
 
 /** Devices list.
  */
 typedef struct {
 	list_head_t dev;
+	int version;
+	int delall;
 } veth_param;
 
 /** Create/remove veth devices for VE.
@@ -50,10 +62,23 @@ typedef struct {
  * @return		0 on success.
  */
 struct vps_param;
-int vps_setup_veth(vps_handler *h, envid_t veid, struct vps_param *param);
+int vps_setup_veth(vps_handler *h, envid_t veid, dist_actions *actions,
+	char *root, veth_param *veth_add, veth_param *veth_del, int state,
+	int skip);
 int add_veth_param(veth_param *list, veth_dev *dev);
+void free_veth(list_head_t *head);
 void free_veth_param(veth_param *dev);
 int copy_veth_param(veth_param *dst, veth_param *src);
 int merge_veth_list(list_head_t *old, list_head_t *add, list_head_t *del,
 	veth_param *merged);
+void generate_mac(int veid, char *dev_name, char *mac);
+void free_veth_dev(veth_dev *dev);
+veth_dev *find_veth_by_ifname_ve(list_head_t *head, char *name);
+veth_dev *find_veth_configure(list_head_t *head);
+int merge_veth_list(list_head_t *old, list_head_t *add, list_head_t *del,
+	veth_param *merged);
+int parse_hwaddr(const char *str, char *addr);
+int check_veth_param(envid_t veid, veth_param *veth_old, veth_param *veth_new,
+	veth_param *veth_del);
+
 #endif	/* _VETH_H_ */
