@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2006 SWsoft. All rights reserved.
+ * Copyright (C) 2000-2007 SWsoft. All rights reserved.
  *
  * This code is public domain.
  *
@@ -485,7 +485,7 @@ int main(int argc,char** argv)
 
 	if (ip6_addr) {
 		if (cmd == AR_UPDATE)
-			execlp("/usr/sbin/ndsend", "ndsend", ip6_addr, iface, NULL);
+			execlp(SBINDIR "/ndsend", "ndsend", ip6_addr, iface, NULL);
 		exit(0);
 	}
 
