@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000-2006 SWsoft. All rights reserved.
+ *  Copyright (C) 2000-2007 SWsoft. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -179,22 +179,22 @@ int vps_umount(vps_handler *h, envid_t veid, char *root, skipFlags skip)
 		return 0;
 	}
 	if (!(skip & SKIP_ACTION_SCRIPT)) {
-		snprintf(buf, sizeof(buf), "%svps.%s", VPS_CONF_DIR,
-			UMOUNT_PREFIX);
+		snprintf(buf, sizeof(buf), "%s%d.%s", VPS_CONF_DIR,
+			veid, UMOUNT_PREFIX);
 		for (i = 0; i < 2; i++) {
 			if (run_pre_script(veid, buf)) {
 				logger(-1, 0, "Error executing umount script %s",
 					buf);
 				return VZ_ACTIONSCRIPT_ERROR;
 			}
-			snprintf(buf, sizeof(buf), "%s%d.%s", VPS_CONF_DIR,
-				veid, UMOUNT_PREFIX);
+			snprintf(buf, sizeof(buf), "%svps.%s", VPS_CONF_DIR,
+				UMOUNT_PREFIX);
 		}
 	}
 	if (!(ret = fsumount(veid, root)))
 		logger(0, 0, "VE is unmounted");
 
-	return 0;
+	return ret;
 }
 
 int vps_set_fs(fs_param *g_fs, fs_param *fs)
