@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000-2006 SWsoft. All rights reserved.
+ *  Copyright (C) 2000-2007 SWsoft. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -505,6 +505,17 @@ static int set(vps_handler *h, envid_t veid, vps_param *g_p, vps_param *vps_p,
 	char *dist_name;
 
 	ret = 0;
+	
+	if (!list_empty(&cmd_p->res.veth.dev) ||
+	    !list_empty(&cmd_p->del_res.veth.dev))
+	{
+		ret = check_veth_param(veid, &vps_p->res.veth, &cmd_p->res.veth,
+			&cmd_p->del_res.veth);
+		if (ret) {
+			cmd_p->opt.save = NO;
+			return ret;
+		}
+	}
 
 	cmd_p->g_param = g_p;
 	if (cmd_p->opt.apply_cfg_map == APPCONF_MAP_NAME) {
@@ -652,6 +663,8 @@ static int exec(vps_handler *h, int action, envid_t veid, char *root, int argc,
 	int i, ret;
 	int len, totallen = 0;
 
+	/* disable logging on terminal  */
+	set_log_verbose(0);
 	mode = MODE_BASH;
 	if (action == ACTION_EXEC3) {
 		arg = argv;
