@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000-2006 SWsoft. All rights reserved.
+ *  Copyright (C) 2000-2007 SWsoft. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -55,7 +55,7 @@ void usage()
 	struct mod_action mod;
 
 	version();
-	fprintf(stdout, "Copyright (C) 2000-2006 SWsoft.\n");
+	fprintf(stdout, "Copyright (C) 2000-2007 SWsoft.\n");
 	fprintf(stdout, "This program may be distributed under the terms of the GNU GPL License.\n\n");
 	fprintf(stdout, "Usage: vzctl [options] <command> <veid> [parameters]\n"
 "vzctl destroy | mount | umount | stop | status | enter <veid>\n"
@@ -75,8 +75,7 @@ void usage()
 "   [--noatime yes|no] [--capability <name>:on|off ...]\n"
 "   [--devices b|c:major:minor|all:r|w|rw]\n"
 "   [--devnodes device:r|w|rw|none]\n"
-"   [--veth_add dev_name,dev_addr,dev_name_ve,dev_addr_ve]\n"
-"   [--veth_del dev_name]\n"
+"   [--netif_add <ifname[,mac,host_ifname,host_mac]]>] [--netif_del <ifname>]\n"
 "   [--applyconfig <name>] [--applyconfig_map <name>]\n"
 "   [--features <name:on|off>] [--name <vename>]\n");
 
@@ -103,13 +102,15 @@ int main(int argc, char *argv[], char *envp[])
 {
 	int action = 0;
 	int verbose = 0;
+	int verbose_tmp;
+	int verbose_custom = 0;
 	int quiet = 0;
 	int veid, ret, skiplock = 0;
 	char buf[256];
 	vps_param *gparam = NULL, *vps_p = NULL, *cmd_p = NULL;
 	const char *action_nm;
 	struct sigaction act;
-	char *name = NULL;
+	char *name = NULL, *opt;
 
 	_proc_title = argv[0];
 	_proc_title_len = envp[0] - argv[0];
@@ -124,14 +125,32 @@ int main(int argc, char *argv[], char *envp[])
 	sigaction(SIGPIPE, &act, NULL);
 
 	while (argc > 1) {
-		if (!strcmp(argv[1], "--verbose")) {
-			verbose = 2;
-		} else if (!strcmp(argv[1], "--quiet"))
+		opt = argv[1];
+
+		if (!strcmp(opt, "--verbose")) {
+			if (argc > 2 &&
+			    !parse_int(argv[2], &verbose_tmp))
+			{
+				verbose += verbose_tmp;
+				argc--; argv++;
+			} else {
+				verbose++;
+			}
+			verbose_custom = 1;
+		} else if (!strncmp(opt, "--verbose=", 10)) {
+			if (parse_int(opt + 10, &verbose_tmp)) {
+				fprintf(stderr, "Invalid value for"
+					" --verbose\n");
+				exit(VZ_INVALID_PARAMETER_VALUE);
+			}
+			verbose += verbose_tmp;
+			verbose_custom = 1;
+		} else if (!strcmp(opt, "--quiet"))
 			quiet = 1;
-		else if (!strcmp(argv[1], "--version")) {
+		else if (!strcmp(opt, "--version")) {
 			version();
 			exit(0);
-		} else if (!strcmp(argv[1], "--skiplock"))
+		} else if (!strcmp(opt, "--skiplock"))
 			skiplock = YES;
 		else
 			break;
@@ -229,8 +248,18 @@ int main(int argc, char *argv[], char *envp[])
 		goto error;
 	}
 	init_log(gparam->log.log_file, veid, gparam->log.enable != NO,
-		verbose ? gparam->log.level + 2 : gparam->log.level,
-		quiet, "vzctl");
+		gparam->log.level, quiet, "vzctl");
+	/* Set verbose level from global config if not overwriten
+	   by --verbose 
+	*/
+	if (!verbose_custom && gparam->log.verbose != NULL) {
+		verbose = *gparam->log.verbose;
+		verbose_custom = 1;
+	}
+	if (verbose < -1)
+		verbose = -1;
+	if (verbose_custom)
+		set_log_verbose(verbose);
 	if ((ret = parse_action_opt(veid, action, argc, argv, cmd_p,
 		action_nm)))
 	{
