@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000-2006 SWsoft. All rights reserved.
+ *  Copyright (C) 2000-2007 SWsoft. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -1073,6 +1073,39 @@ char *invert_ip(char *ips)
 	return tmp;
 }
 
+static int get_run_ve_proc2(int update)
+{
+	FILE *fp;
+	struct Cveinfo ve;
+	char buf[16384];
+	int res, veid;
+
+	if ((fp = fopen(PROCVEINFO, "r")) == NULL) {
+		fprintf(stderr, "Unable to open %s\n", PROCVEINFO);
+		return 1;
+	}
+	while (!feof(fp)) {
+		if (fgets(buf, sizeof(buf), fp) == NULL)
+			break;
+		res = sscanf(buf, "%d", &veid);
+		if (res != 1 || veid == 0)
+			continue;
+		if (!check_veid_restr(veid))
+			continue;
+		memset(&ve, 0, sizeof(struct Cveinfo));
+		ve.veid = veid;
+		ve.status = VE_RUNNING;
+		if (update)
+			update_ve(veid, ve.ip, ve.status);
+		else
+			add_elem(&ve);
+	}
+	if (!update)
+		qsort(veinfo, n_veinfo, sizeof(struct Cveinfo), id_sort_fn);
+	fclose(fp);
+	return 0;
+}
+
 static int get_run_ve_proc(int update)
 {
 	FILE *fp;
@@ -1082,8 +1115,7 @@ static int get_run_ve_proc(int update)
 	int res, veid, classid, nproc;
 
 	if ((fp = fopen(PROCVEINFO, "r")) == NULL) {
-		fprintf(stderr, "Unable to open %s\n", PROCVEINFO);
-		return 1;
+		return get_run_ve_proc2(update);
 	}
 	memset(&ve, 0, sizeof(struct Cveinfo));
 	while (!feof(fp)) {
@@ -1159,7 +1191,7 @@ static int get_run_ve_ioctl(int update)
 	void *buf = NULL;
 	int i; 
 
-	vzctlfd = open("/dev/vzctl", O_RDWR);
+	vzctlfd = open(VZCTLDEV, O_RDWR);
 	if (vzctlfd < 0) 
 		goto error;
 	veid.num = 256;
@@ -1323,7 +1355,7 @@ int get_ves_la()
 {
 	int i;
 	
-	if ((vzctlfd = open("/dev/vzctl", O_RDWR)) < 0)
+	if ((vzctlfd = open(VZCTLDEV, O_RDWR)) < 0)
 		return 1;
 	for (i = 0; i < n_veinfo; i++) {
 		if (veinfo[i].hide)
